@@ -29,7 +29,7 @@ from neo4jrestclient.query import (
 from neo4jrestclient.request import Request
 from neo4jrestclient.exceptions import (NotFoundError, StatusException,
                                         TransactionException)
-from neo4jrestclient.traversals import TraversalDescription
+from neo4jrestclient.traversals import TraversalDescription, GraphTraversal
 from neo4jrestclient.utils import (PY2, text_type, smart_quote, string_types,
                                    unquote)
 
@@ -94,9 +94,7 @@ class GraphDatabase(object):
             self.VERSION = response_json.get('neo4j_version', None)
             if self.VERSION:
                 self._auth.update({'version': self.VERSION})
-            self.extensions = ExtensionsProxy(self._extensions,
-                                              auth=self._auth,
-                                              cypher=self._cypher)
+            self._extensions_cache = None
             self.nodes = NodesProxy(self._node, self._reference_node,
                                     self._node_index,
                                     auth=self._auth, cypher=self._cypher)
@@ -110,12 +108,36 @@ class GraphDatabase(object):
                                                     self._relationship_index,
                                                     auth=self._auth,
                                                     cypher=self._cypher)
-            self.Traversal = self._get_traversal_class()
+            self.Traversal = GraphTraversal
             try:
                 self._batch = response_json["batch"]
             except KeyError:
                 self._batch = "%sbatch" % self.url
 
+    def __eq__(self, obj):
+        return (
+            self._relationship_index == obj._relationship_index
+            and self._relationship == obj._relationship
+            and self._node == obj._node
+            and self._labels == obj._labels
+            and self._labels_list == obj._labels_list
+            and self._node_index == obj._node_index
+            and self._reference_node == obj._reference_node
+            and self._extensions_info == obj._extensions_info
+            and self._extensions == obj._extensions
+            and self._cypher == obj._cypher
+            and self._transaction == obj._transaction
+            and self._batch == obj._batch
+        )
+
+    def _get_extensions(self):
+        if not self._extensions_cache:
+            self._extensions_cache = ExtensionsProxy(self._extensions,
+                                                     auth=self._auth,
+                                                     cypher=self._cypher)
+        return self._extensions_cache
+    extensions = property(_get_extensions)
+
     def _get_reference_node(self):
         warnings.warn("Deprecated, the reference node is not needed anymore",
                       DeprecationWarning)
@@ -138,62 +160,9 @@ class GraphDatabase(object):
     def traversal(self):
         return TraversalDescription(auth=self._auth, cypher=self._cypher)
 
-    def _get_traversal_class(self):
-        cls = self
-
-        class Traversal(object):
-            types = None
-            order = None
-            stop = None
-            returnable = None
-            uniqueness = None
-            paginated = None
-            page_size = None
-            time_out = None
-            returns = None
-            is_returnable = None
-            isReturnable = None
-            is_stop_node = None
-            isStopNode = None
-
-            def __init__(self, start_node=None):
-                if start_node is not None and isinstance(start_node, Node):
-                    self.start_node = start_node
-                else:
-                    self.start_node = cls.reference_node
-                is_returnable = self.is_returnable or self.isReturnable
-                is_stop_node = self.is_stop_node or self.isStopNode
-                results = self.start_node.traverse(types=self.types,
-                                                   order=self.order,
-                                                   stop=self.stop,
-                                                   returnable=self.returnable,
-                                                   uniqueness=self.uniqueness,
-                                                   is_stop_node=is_stop_node,
-                                                   is_returnable=is_returnable,
-                                                   paginated=self.paginated,
-                                                   page_size=self.page_size,
-                                                   time_out=self.time_out,
-                                                   returns=self.returns)
-                self._items = results
-                self._index = len(results)
-
-            def __iter__(self):
-                return self
-
-            def __next__(self):
-                if self._index == 0:
-                    raise StopIteration
-                self._index = self._index - 1
-                return self._items[self._index]
-
-            def next(self):
-                return self.__next__()
-
-        return Traversal
-
     def transaction(self, using_globals=True, commit=True, update=True,
                     transaction_id=None, context=None, for_query=False,
-                    rollback=True):
+                    rollback=True, execute=False):
         if transaction_id not in self._transactions:
             transaction_id = len(self._transactions.keys())
         if for_query:
@@ -204,7 +173,8 @@ class GraphDatabase(object):
                 "position": Position,
             }
             tx = QueryTransaction(self, transaction_id, rollback=rollback,
-                                  commit=commit, update=update, types=types)
+                                  commit=commit, update=update, types=types,
+                                  execute=execute)
         else:
             tx = Transaction(self, transaction_id, context or {},
                              commit=commit, update=update)
@@ -214,7 +184,7 @@ class GraphDatabase(object):
         return self._transactions[transaction_id]
 
     def query(self, q, params=None, returns=RAW, tx=None):
-        if self._cypher:
+        if self._cypher or self._transaction:
             types = {
                 "node": Node,
                 "relationship": Relationship,
@@ -222,6 +192,11 @@ class GraphDatabase(object):
                 "position": Position,
             }
             tx = Transaction.get_transaction(tx)
+            # The non transactional Cypher endpoint will be removed eventually,
+            # So we create always a transaction per query for Neo4j 2.0+
+            if (tx is None
+                    and self.VERSION and self.VERSION.split(".")[0] >= "2"):
+                tx = self.transaction(for_query=True, execute=True)
             return QuerySequence(self._cypher, self._auth, q=q, params=params,
                                  types=types, returns=returns, tx=tx)
         else:
@@ -234,7 +209,6 @@ class GraphDatabase(object):
                                             cypher=self._cypher,
                                             node=Node)
         return self._labels_list
-
     labels = property(_get_labels)
 
 
@@ -826,7 +800,10 @@ class Base(object):
         property_url = self._dic["property"].replace("{key}", smart_quote(key))
         tx = Transaction.get_transaction(tx)
         if tx:
-            return tx.append(TX_GET, property_url, obj=self)
+            if isinstance(tx, QueryTransaction):
+                return self._dic["data"][key]
+            else:
+                return tx.append(TX_GET, property_url, obj=self)
         response = Request(**self._auth).get(property_url)
         if response.status_code == 200:
             self._dic["data"][key] = response.json()
